/* advice.h: Contains message formats for the messages sent between an */
/*  external controller and the handoff routing daemon. */

#include <sys/types.h>
#include <netinet/in.h>

/* Constants */

/* The Port number for communication from advice giver advice receiver. */

#define ADVICE_REQUEST_PORT 9544
#define ADVICE_REPLY_PORT 9545

/* These indicate what the message is: */
/* For more details on what these commands mean, see the handoff */
/* working doc on the WWW page */

#define USE_THIS_NETWORK (0)
#define USE_ANY_NETWORK  (1)
#define DONT_USE_THIS_NETWORK (2)
#define USE_THIS_BASESTATION (3)
#define USE_ANY_BASESTATION (4)
#define DONT_USE_THIS_BASESTATION (5)

/* If the top bit is set, it's a reply */
#define REPLY_MASK (0x80)

/* So the message format is as follows:  */
/* -----------------                     */
/* | opcode | body |                     */
/* -----------------                     */
/*   1 byte  10 bytes(max)               */

/* If a body field is an interface, it is a null-terminated string */
/* that is the interface namee */
/* If a body field is an in_addr, it follows the same ordering */
/* conventions as in IP */

#define MAX_IF_SIZE (10)

typedef struct _AdviceBodyIf {
	char interface[MAX_IF_SIZE];
} AdviceBodyIf;

typedef struct _AdviceBodyBS {
	struct in_addr addr;
} AdviceBodyBS;

/* Here is a structure that can hold a message */
typedef struct AdviceMsg {
	struct in_addr client;
	u_int opcode;
	union {
		AdviceBodyIf interface;
		AdviceBodyBS bs;
	} body;
} AdviceMsg;

/* Macros for data encapuslation */
#define Advice_MsgClient(msg) ((msg)->client)
#define Advice_MsgOpcode(msg) ((msg)->opcode)
#define Advice_MsgIf(msg) ((msg)->body.interface.interface)
#define Advice_MsgBs(msg) ((msg)->body.bs.addr)

/* Function prototypes */

void ProcessAdviceRequest(char *advicemsg, RouteState * sysStatePtr, IFace * curiPtr);
void ProcessStateRequest(char *statemsg, RouteState * sysStatePtr, IFace * curiPtr);
