/*
 * Copyright (c) 1995, 1996
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* 
 * User-level decapsulation daemon for Daedalus Project's handoff software.
 * Original version written by Srinivasan Seshan (srini@watson.ibm.com)
 * modified heavily and released by Mark Stemm (stemm@cs.berkeley.edu)
 *
 * Daedalus group, Dept. of EECS, U.C. Berkeley.
 * Contact: stemm@cs.berkeley.edu, net@daedalus.cs.berkeley.edu
 */

#ifndef DECAP_H
#define DECAP_H

/* Port on which decapsulation program accepts requests */
#define BSR_PORT	1300

/* Possible requests that a mobile can make */
typedef enum {UNENCAP_ADD, UNENCAP_ENA, UNENCAP_DIS, UNENCAP_DEL, UNENCAP_FORCE_ENA} BSRTypes;

/* Possible states that a mobile can be in */
typedef enum {M_INACTIVE=0, M_BUFFERING, M_FORWARDING} Mobile_States;

/* Default ttl for multicast group from home agent */
#define DEF_TTL 4

/* Maximum number of mobiles per base station */
#define MAX_MHS 20

/* The format of a base station request message */
typedef struct BSReq {
	struct in_addr recipientAddr; /* The address of the recipient of the message */
	BSRTypes type;
	struct in_addr locAddr; /* The local address of the mobile */
	struct in_addr realAddr; /* The home address of the mobile */
	struct in_addr multiAddr; /* The multicast address used to send packets from home agent */
	struct mobileip_ids ids;  /* The last few packets seen by the mobile */
	int hdrOnly;            /* Whether to forward/buffer headers or full packets */
} BSReq;

typedef struct MobileState {
	Mobile_States state; /* What action the base station should be doing with respect to this base station */
	int hdrOnly;         /* Whether to send headers or full packets */
	Socket *cdPtr;       /* Socket for communication to mobile */
	char ln[20];         /* Mobile's local (wireless) name */
	char rn[20];         /* Mobile's real (wired) name */
	char mn[20];         /* Mobile's multicast name */
} MobileState;

typedef struct _SocketList {
	Socket *sock;
	struct _SocketList *nextSock;
} SocketList;

typedef struct DecapdState {
	struct in_addr wiAddr;
	struct in_addr wlAddr;
	Socket *sdPtr;
	fd_set fdvar;
	int ctrlFD;
	int routes;
	SocketList connList; /* Linked list of active connections */
	SocketList *lastSock; /* Pointer to end of list */
} DecapdState;

#endif /* #ifdef DECAP_H */

