/* handoffctlr.h: Contains message formats for the messages sent */
/* between the Handoff Controller and other people. */

#include <sys/types.h>
#include <netinet/in.h>

/* Constants */

#define HANDOFFCTLR_REQUEST_PORT 9541
#define HANDOFFCTLR_REPLY_PORT 9542
#define PROXY_PORT 9543

/* These indicate what the message is: */
/* For more details on what these commands mean, see the handoff */
/* working doc. */

#define LIST_ALL_NETWORKS 0
#define LIST_ACTIVE_NETWORK 1
#define LIST_ALL_BASESTATIONS 2
#define LIST_ACTIVE_BASESTATION 3
#define LIST_USED_NETWORKS 4
#define LIST_USED_BASESTATIONS 5
#define LIST_DONTUSED_NETWORKS 6
#define LIST_DONTUSED_BASESTATIONS 7

/* If the top bit is set, it's a reply */
#define REPLY_MASK (0x80)

/* So the message format is as follows:                          */
/* ------------------------------------------------------------- */
/* | client |  opcode | num_body | body1 | body2 | ... | bodyN | */
/* ------------------------------------------------------------- */
/*   4 bytes   1 byte   1 byte   ? bytes ? bytes        ? bytes  */

/* For "networks..." messages, a network is the Unix device name(efp, */
/*   irp, wlp, ...) as a null-terminated string.  */
/* For "basestations..."messages, a body is an in_addr in network order */

/* The size of the message without any bodies */
#define Handoffctlr_BaseMsgSize (6)
#define Handoffctlr_MaxIfSize (10)

typedef char String[100];

/* Here is a structure that can hold a message */
/* Only one of (body_ifs, body_addrs) is active at one time */
typedef struct _HandoffctlrMsg {
	struct in_addr client;
	u_char opcode;
	u_char numBody;
	String bodyIfs[100];
	struct in_addr bodyAddrs[100];
} HandoffctlrMsg;

/* Here are some utility functions that covert between byte arrays and HandoffctlrMsg structures */
int Handoffctlr_MsgToString(char *string, HandoffctlrMsg *msg);
void Handoffctlr_StringToMsg(HandoffctlrMsg *msg, char *string);

/* Data Sbstraction Macros for getting to fields of message */
#define Handoffctlr_MsgClient(msg) ((msg)->client)
#define Handoffctlr_MsgOpcode(msg) ((msg)->opcode)
#define Handoffctlr_MsgNumBody(msg) ((msg)->numBody)
#define Handoffctlr_MsgIfs(msg) ((msg)->bodyIfs)
#define Handoffctlr_MsgAddrs(msg) ((msg)->bodyAddrs)
#define Handoffctlr_MsgIf(msg, i) ((msg)->bodyIfs[i])
#define Handoffctlr_MsgAddr(msg, i) ((msg)->bodyAddrs[i])
