/*************************************************************
 *
 * Copyright 1992 Regents of the University of California
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * $Header: /usr/local/daedalus/include/RCS/link.h,v 1.1 1996/11/06 22:00:31 hari Exp $
 *
 *************************************************************
 */

#ifndef	_LINKLIST_H_
#define	_LINKLIST_H_

#include "localtypes.h"

#define	LINK_LIST_END	(void*)0

class Link {
    Link*	prev;
    Link*	next;
    void*	top;
    friend class	ListHeader;
  public:
    void	InitElement (void*);
    void	Insert (Link*);
    void	Remove ();
    Link*	LPrev () const		{return (prev);}
    Link*	LNext () const		{return (next);}
    Link*	Before () const		{return (prev);}
    Link*	After() 		{return (this);}
    void*	Element () const	{return (top);}
    void*	Prev () const		{return (prev->top);}
    void*	Next () const		{return (next->top);}
};

typedef void (*LinkFuncPtr)(Link*);

class ListHeader {
  protected:
    Link	list;
  public:
    void	Init ();
    void*	First () const	{return (list.Next());}
    void*	Last ()	const	{return (list.Prev());}
    Link*	LFirst () const	{return (list.LNext());}
    Link*	LLast () const	{return (list.LPrev());}
    Link*	Front () 	{return (&list);}
    Link*	Rear ()	const	{return (LLast ());}
    Boolean	IsEmpty () const {return (list.next == &list);}
    Boolean	IsAtEnd (Link* l) {return (list.prev == l);}
    void	InsertList (Link* l);
    void	ForAll (LinkFuncPtr);
};

/***********************************************************************
 *
 *	Link::Insert
 *
 *	Insert an element into a list.  This must be passed both
 *	the item's address AND a pointer to the actual link field.
 *	This allows elements which are in multiple lists to be
 *	handled cleanly.
 *
 ***********************************************************************
 */
inline
void
Link::Insert (Link* where)
{
    this->next = where->next;
    this->prev = where;
    this->prev->next = this;
    this->next->prev = this;
}

/***********************************************************************
 *
 *	Link::Remove
 *
 *	This will remove an element from the list it's on.  As with
 *	other list routines, it's designed to accommodate elements
 *	on two or more lists at the same time.
 *
 ***********************************************************************
 */
inline
void
Link::Remove ()
{
    this->prev->next = this->next;
    this->next->prev = this->prev;
}

#endif	/* _LINKLIST_H_ */
