/*************************************************************
 *
 * Copyright 1991 Regents of the University of California
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * $Header: /usr/local/daedalus/include/RCS/longInt.h,v 1.1 1996/11/06 22:00:31 hari Exp $
 *
 *************************************************************
 */

#define _INT64_HARDWARE_
#ifndef _INT64_H_
#define _INT64_H_

typedef unsigned int uint32;
typedef unsigned short uint16;
typedef unsigned char uint8;

#ifdef _INT64_HARDWARE_
/*
 * Add something here if a machine has 64 bit integer support in hardware.
 * Must provide low(x), high(x) to get the low and high 32 bit sections,
 * respectively.
 */
typedef unsigned long long uint64;
#else

#define	DOUBLE_2_TO_32		((double)(1 << 16) * (double)(1 << 16))
class uint64 {
    uint32 lowPart;
    uint32 highPart;
  public:
    uint64 (unsigned int i1, unsigned int i2) {lowPart = i2; highPart = i1;}
    uint64 (unsigned int i) {lowPart = i; highPart = 0;}
    uint64 (int i)	{lowPart = (unsigned int)i; highPart = 0;}
    uint64 ()		{lowPart = 0; highPart = 0;}
    uint32 low()	{return lowPart;}
    uint32 high()	{return highPart;}
    operator uint32()	{return lowPart;}
    operator double()	{return ((double)lowPart + ((double)highPart *
						    DOUBLE_2_TO_32));}
    uint64 (double d)	{highPart = (uint32)(d / DOUBLE_2_TO_32);
			 lowPart = (uint32)(d - ((double)highPart *
						 DOUBLE_2_TO_32));}
    friend uint64 operator+ (uint64,uint64);
    friend uint64 operator* (uint64&,uint32);
    friend uint64 operator* (uint32,uint64);
    friend uint64 operator- (uint64,uint64);
    friend uint32 operator==(uint64,uint64);
    friend uint32 operator!=(uint64,uint64);
    friend uint32 operator>=(uint64,uint64);
    friend uint32 operator<=(uint64,uint64);
    friend uint32 operator> (uint64,uint64);
    friend uint32 operator< (uint64,uint64);
    friend uint64 operator+=(uint64,uint64);
    friend uint64 operator-=(uint64,uint64);
    friend uint64 operator/ (uint64,uint64);
#if 0
    friend uint64 operator% (uint64,uint32);
#endif
};

/***********************************************************************
 *
 *	Addition and subtraction for 64-bit integers
 *
 ***********************************************************************
 */
inline
uint64
operator+(uint64 i1, uint64 i2)
{
    uint64 result;

    result.lowPart = i1.lowPart + i2.lowPart;
    if (result.lowPart < i1.lowPart) {
	result.highPart = 1;
    } else {
	result.highPart = 0;
    }
    result.highPart += (i1.highPart + i2.highPart);

    return (result);
}

inline
uint64
operator-(uint64 i1, uint64 i2)
{
    uint64 result;

    result.lowPart = i1.lowPart - i2.lowPart;
    result.highPart = i1.highPart - i2.highPart;
    if (i1.lowPart < i2.lowPart) {
	result.highPart -= 1;
    }
    return (result);
}

/***********************************************************************
 *
 *	64-bit comparison operators
 *
 ***********************************************************************
 */
inline
uint32
operator==(uint64 i1, uint64 i2)
{
    return ((i1.lowPart == i2.lowPart) && (i1.highPart == i2.highPart));
}

inline
uint32
operator!=(uint64 i1, uint64 i2)
{
    return (!(i1 == i2));
}

inline
uint32
operator>(uint64 i1, uint64 i2)
{
    if (i1.highPart > i2.highPart) {
	return (2 > 1);
    } else if (i1.highPart < i2.highPart) {
	return (1 > 2);
    } else {
	return (i1.lowPart > i2.lowPart);
    }
}

inline
uint32
operator>=(uint64 i1, uint64 i2)
{
    return ((i1 == i2) || (i1 > i2));
}

inline
uint32
operator<(uint64 i1, uint64 i2)
{
    return (i2 > i1);
}

inline
uint32
operator<=(uint64 i1, uint64 i2)
{
    return (i2 >= i1);
}

inline
uint64
operator/ (uint64 i1, uint64 i2)
{
    return ((uint64)((double)i1 / (double)i2));
}

#endif _INT64_HARDWARE_

#endif _INT64_H_
