/***********************************************************************
 *
 * Copyright 1991 Regents of the University of California
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * $Header: /usr/local/daedalus/include/RCS/migTrace.h,v 1.1 1996/11/06 22:00:31 hari Exp $
 *
 ***********************************************************************
 */

#ifndef	_MIGTRACE_H_
#define	_MIGTRACE_H_

#ifndef __cplusplus
#include "list.h"
#else
#include "linklist.h"
#endif

#ifndef TRUE
#define	TRUE			1
#endif

#ifndef FALSE
#define	FALSE			0
#endif

#define MIGRATION_FIRST_YEAR	84
#define	MAX_LINE		(400)
#define MAX_INTERLEAVED		(4000)
#define MAX_SEQID_LENGTH	(12)
#define MAX_FILE_NAME_LENGTH	(150)

#define DEVICE_CRAY_DISK	'y'
#define DEVICE_TAPE_SILO	's'
#define DEVICE_TAPE_MANUAL	't'
#define DEVICE_DISK_FARM	'r'
#define DEVICE_UNKNOWN		'?'

#define MIGRATION_START_TIME	'S'	/* a record with S as first char
					 * contains the trace start time */
#define MIGRATION_COMMENT	'X'	/* a record with X as first char
					 * is a comment record */
#define MIGRATION_PRECISION	'P'	/* a record with P as first char
					 * defines time precision for trace */

#define MIGRATION_WRITE		0x1	/* bit set if write, clear if read */
#define MIGRATION_SAME_UID	0x2	/* bit set if uid same as prev. rec */
#define	MIGRATION_SAME_SIZE	0x4	/* bit set if size same as prev rec */
#define MIGRATION_ERROR		0x10	/* bit set for migration error */
#define MIGRATION_TIMED_OUT	0x20	/* migration never finished */
#define MIGRATION_SYS_RESTARTED	0x40	/* sys restarted before mig done */

#define	MIGRATION_DB_MAGIC_SEG_0 0xface	/* magic # for segment 0 */
#define	MIGRATION_DB_MAGIC_SEG_OTHER 0xdeed /* magic # for other segments */

/*
 * These are flags indicating what type of error occurred.  These are
 * for errors that the mass storage controller reports (as opposed to
 * timeouts and system restarts which are never reported).   These
 * are mutually exclusive, so they may are never XORed together
 */
#define MIGRATION_ERROR_MASK		0xff00	/* mask off all but errors */
#define MIGRATION_ERR_UNKNOWN		0x100	/* none of the following */
#define MIGRATION_ERR_NO_FILE		0x200	/* no file found */
#define MIGRATION_ERR_ACCESS		0x300	/* access error */
#define	MIGRATION_ERR_QUEUE_FULL	0x400	/* queue for device full */
#define MIGRATION_ERR_BAD_USER		0x500	/* bad user? */
#define	MIGRATION_ERR_NO_SPACE		0x600	/* no space available */
#define MIGRATION_ERR_BAD_DATA		0x700	/* failed data check */
#define	MIGRATION_ERR_BAD_NAME		0x800	/* ??? */
#define MIGRATION_ERR_USER_QUIT		0x900	/* user killed operation? */
#define MIGRATION_ERR_CREATE		0xa00	/* file creation failed */
#define MIGRATION_ERR_DIRECTORY		0xb00	/* attempt to operate on dir */
#define MIGRATION_ERR_NOT_DIRECTORY	0xc00	/* not directory; should be */
#define	MIGRATION_ERR_BAD_MEDIUM	0xd00	/* bad medium? */
#define	MIGRATION_ERR_TOO_BIG		0xe00	/* file too large to migrate */
#define	MIGRATION_ERR_SHUTDOWN		0xf00	/* system was shut down */
#define MIGRATION_ERR_FILE_TRUNC	0x1000	/* file truncated (no spc?) */
#define MIGRATION_ERR_WPWD		0x4000	/* ??? */
#define MIGRATION_ERR_RPWD		0x4100	/* ??? */
#define MIGRATION_ERR_HOSTRW		0x4200	/* ??? */

#define	MIGRATION_DB_SEG_SIZE		300	/* amt in one page of data */

#define MAX_ERROR_TIME		(3600)	/* error after 1 hour */

#ifndef __cplusplus
typedef unsigned int	uint32;
typedef	long long	uint64;
#else
#include "longInt.h"
#endif

typedef struct MigrationInfo {
    uint32	done;			/* is this record in use? */
    uint32	startTime;		/* (absolute) time of I/O start */
    uint32	xferLatency;		/* time required for transfer */
    uint32	pickAndSeekLatency;	/* latency to mount and seek tape */
    uint64	fileSize;		/* file size in bytes */
    uint32	userId;			/* user ID that requested xfer */
    uint32	flags;
#ifndef __cplusplus
    List_Links 	activeNext;		/* -> next active I/O in chain */
    struct	MigrationInfo *top1;	/* -> top of this structure */
    List_Links	timeNext;		/* -> next I/O in time sequence */
    struct	MigrationInfo *top2;	/* -> top of this structure */
#else
    Link	activeNext;
    Link	timeNext;
#endif
    char	fromDevice;		/* device the xfer was from */
    char	toDevice;		/* device the xfer was to */
    char	seqId[MAX_SEQID_LENGTH]; /* sequence ID (for assembling rec) */
    char	userName[32];		/* user name */
    char	bitFileName[MAX_FILE_NAME_LENGTH];
    char	unixFileName[MAX_FILE_NAME_LENGTH];
} MigrationInfo;

typedef struct ListMigInfo {
#ifndef __cplusplus
    List_Links		links;
    MigrationInfo	*top;
#else
    Link		links;
#endif
} ListMigInfo;

typedef struct MigrationDbHdr {
    unsigned short	numReads;
    unsigned short	numWrites;
    uint32		recordLength;
    uint64		fileSize;
} MigrationDbHdr;

typedef struct MigrationDbAccess {
    unsigned short	flags;
    unsigned short	userId;
    uint32		refTime;
} MigrationDbAccess;

#endif	/* _MIGTRACE_H_ */
