#define SOCK_SIDE_CLIENT			1<<1
#define SOCK_SIDE_SERVER			1<<2

#define SOCK_CTRL_TIMEOUT_SEC			0
#define SOCK_CTRL_TIMEOUT_USEC			500000
#define SOCK_MIN_USLEEP				1500
#define SOCK_DEFAULT_BUF_SIZE 	        	32768
#define SOCK_MAX_MULTICAST_HOSTS		4
#define SOCK_MAX_HOST_NAME_LEN			80
#define SOCK_MAX_HANDOFF_HOSTS			4
#define SOCK_MAX_CTRL_HOSTS			2


#define SOCK_STATE_ALLOCATED			1
#define SOCK_STATE_LISTENING			2
#define SOCK_STATE_CONNECTED			3
#define SOCK_STATE_ACCEPTED			4
#define SOCK_STATE_HANDOFF			5
#define SOCK_STATE_CLOSED			6
#define SOCK_STATE_CLOSING			7

#define SOCK_CTRL_INIT_HANDOFF			1
#define SOCK_CTRL_DO_HANDOFF			2
#define SOCK_CTRL_SET_RATE			3
#define SOCK_CTRL_SET_REMOTE			4
#define SOCK_CTRL_SET_REMOTE_CTRL		5
#define SOCK_CTRL_SET_REMOTE_CTRL_PORT		6
#define SOCK_CTRL_NEW_CTRL_CHANNEL		7
#define SOCK_CTRL_CLOSE_CHANNEL 		8
#define SOCK_CTRL_CLOSE_CTRL_CHANNEL 		9


typedef struct SockNSRequestHdr {
  int reqType;
  int size;
} SockNSRequestHdr;

typedef struct SockCtrlInitHandoff {
  char hostname[SOCK_MAX_HOST_NAME_LEN];
} SockCtrlInitHandoff;

typedef struct SockCtrlDoHandoff {
  char hostname[SOCK_MAX_HOST_NAME_LEN];
} SockCtrlDoHandoff;

typedef struct SockCtrlSetRate {
  int rate;
} SockCtrlSetRate;

typedef struct SockCtrlSetRemote {
  Port port;
} SockCtrlSetRemote;

typedef struct SockCtrlSetRemoteCtrl {
  Port port;
} SockCtrlSetRemoteCtrl;

typedef struct SockCtrlCloseChannel {
  char hostname[SOCK_MAX_HOST_NAME_LEN];
} SockCtrlCloseChannel;

typedef struct SockCtrlCloseCtrlChannel {
  char hostname[SOCK_MAX_HOST_NAME_LEN];
} SockCtrlCloseCtrlChannel;
  
typedef struct SockCtrlNewCtrlChannel {
  int protocol;
  char hostname[SOCK_MAX_HOST_NAME_LEN];
  int port;
} SockCtrlNewCtrlChannel;
  
typedef struct SockCtrlPkt {
  int reqType;
  union {
    SockCtrlInitHandoff 	initHandoff;
    SockCtrlDoHandoff		doHandoff;
    SockCtrlSetRate		setRate;
    SockCtrlSetRemote		setRemote;
    SockCtrlSetRemoteCtrl	setRemoteCtrl;
    SockCtrlNewCtrlChannel	newCtrlChannel;
    SockCtrlCloseChannel	closeChannel;
    SockCtrlCloseChannel	closeCtrlChannel;
  } data;
} SockCtrlPkt;

typedef struct Socket {
  int fd;
  int compatible;
  int state;
  int side;
  int type;
  int protocol;
  char remoteName[SOCK_MAX_MULTICAST_HOSTS + SOCK_MAX_CTRL_HOSTS][SOCK_MAX_HOST_NAME_LEN];
  Port remotePort;
  int rRecvBufSize[SOCK_MAX_MULTICAST_HOSTS];
  int rXmitBufSize[SOCK_MAX_MULTICAST_HOSTS];
  Address rSin;
  char localName[SOCK_MAX_HOST_NAME_LEN];
  Port localPort;
  int lRecvBufSize;
  int lXmitBufSize;
  Address lSin;
  struct timeval timeout, *toutPtr;
  int xmitRate;
  struct timeval lastPktTime;
  int lastPktSize;
  int broadcast;
  int mobile;
  int ctrl;
  int block;
  struct Socket *localCtrlChannel;
  struct Socket *ctrlChannel[SOCK_MAX_MULTICAST_HOSTS + SOCK_MAX_CTRL_HOSTS];
  int activeHosts[SOCK_MAX_MULTICAST_HOSTS + SOCK_MAX_CTRL_HOSTS];
} Socket;

extern Sock_Status 	Sock_RecvCtrl(Socket *ctrlSdPtr, Socket *sdPtr);
extern Sock_Status 	Sock_SendCtrl(Socket *ctrlSdPtr, SockCtrlPkt *ctrlPkt);
extern Sock_Status 	Sock_CtrlSetAddr(Socket *ctrlSdPtr, Port port);
extern Sock_Status 	Sock_CtrlSetCtrlAddr(Socket *ctrlSdPtr, Port port);
extern Sock_Status 	Sock_CtrlSetCtrlPort(Socket *ctrlSdPtr, Port port);
extern char 		*BroadcastAddr(char *normAddr);
extern int 		IsIPAddr(char *id, int length);

