/*
 * resource.h --
 *
 *	Declarations of ...
 *
 * Copyright 1990 Regents of the University of California
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * $Header: /usr/local/daedalus/include/RCS/resource.h,v 1.1 1996/11/06 22:00:31 hari Exp $ SPRITE (Berkeley)
 */

#include "mycompat.h"
#include "queue.h"
#include "event.h"
#include "schedule.h"
#include "semaphore.h"

#ifndef RESOURCE_H
#define RESOURCE_H

#define MAX_NAME_LEN	80
typedef char Name[MAX_NAME_LEN];

typedef enum {QUEUED, BEGIN_USE, END_USE} RequestEventType;

typedef struct {
    /*
     * State information.
     */
    Name	 name;
    int		 maxCapacity;	/* max number of customers in service */
    int		 capacityInUse;
    Sema	 sema;
    int		 queueLen;
    SimTime	 time;
    int		 numRequest;

    /*
     * Time average statistics.
     */
    SimTime	 idleTime;
    SimTime	 busyTime;
    SimTime	 waitTime;
    double	 sumWeighedQueueLen;
    double	 sumWeighedUtil;

    /*
     * State observed by arrivals.
     */
    SimTime	 lastObservedTime;
    int		 numObservingDeparture;
    int		 sumObservedIdle;
    int		 sumObservedQueueLen;
    SimTime	 sumObservedRemainingServiceTime;

    int		 userData;
} Resource;

typedef struct {
    QUEUEtype	 link;
    Name	 name;
    int		 dim;
    int 	*bounds;
    Resource   **resource;
} ResourceGroup;

#define MAX_EVENT_NUM	1024

typedef struct {
    int		 reqID;
    int		 reqType;
    SimTime	 startTime;

    SimTime	 sumQueuedTime;
    SimTime	 sumUseTime;
    SimTime	 time;
    int		 queueNum;
    int		 useNum;
} Request;

Cextern void	InitResource ();
Cextern void	ResetResourceStat();
Cextern Resource	*CreateResource(); /* name queue */
Cextern Resource	*CreateMultiResource(); /* name queue */
Cextern Resource **CreateMultiNResource(); /* name maxCap queue dim bounds...*/
Cextern void	BeginResourceUse(); /* resource request */
Cextern void	EndResourceUse(); /* resource request */
Cextern void	UseResource(); /* resource request timeInterval*/
Cextern int	ResourceLoad(); /* resource */
Cextern void	PrintResourceStat(); /* */
Cextern void	InitRequest(); /* request requestType */
Cextern void	BegRequest(); /* request, reqType */
Cextern void	EndRequest(); /* request */
Cextern void	PrintRequestStat(); /* */

#endif RESOURCE_H
