#define SOCK_SIDE_CLIENT                        1<<1
#define SOCK_SIDE_SERVER                        1<<2

#define SOCK_CTRL_TIMEOUT_SEC                   0
#define SOCK_CTRL_TIMEOUT_USEC                  500000
#define SOCK_MIN_USLEEP                         1500
#define SOCK_DEFAULT_BUF_SIZE                   32768
#define SOCK_DEFAULT_TTL                        4
#define SOCK_MAXHOSTNAME                        1024

#define SOCK_NOT_BROADCAST              0
#define SOCK_BROADCAST                  1
#define SOCK_NOT_MOBILE                 0
#define SOCK_MOBILE                     1
#define SOCK_DONT_USE_CTRL            	0
#define SOCK_USE_CTRL                   1
#define SOCK_NOT_CTRL                   0
#define SOCK_CTRL                       1
#define SOCK_COMPATIBLE                 1
#define SOCK_NOT_COMPATIBLE             0
#define SOCK_BLOCK                      1
#define SOCK_NON_BLOCK                  0

#define SOCK_STATE_ALLOCATED                    1
#define SOCK_STATE_LISTENING                    2
#define SOCK_STATE_CONNECTED                    3
#define SOCK_STATE_ACCEPTED                     4
#define SOCK_STATE_HANDOFF                      5
#define SOCK_STATE_CLOSED                       6
#define SOCK_STATE_CLOSING                      7

#define SOCK_CTRL_INIT_HANDOFF                  1
#define SOCK_CTRL_DO_HANDOFF                    2
#define SOCK_CTRL_SET_RATE                      3
#define SOCK_CTRL_SET_REMOTE                    4
#define SOCK_CTRL_SET_REMOTE_CTRL               5
#define SOCK_CTRL_SET_REMOTE_CTRL_PORT          6
#define SOCK_CTRL_NEW_CTRL_CHANNEL              7
#define SOCK_CTRL_FAREWELL                 	8
#define SOCK_CTRL_CLOSE_CTRL_CHANNEL            9
#define SOCK_CTRL_HELLO                 	10
#define SOCK_CTRL_ACK							11 /* A Socket-library-layer ack */

#define SOCK_UNAVAIL                            0x01 /*slot in cache is used */
#define SOCK_AVAIL                              0x10 /* usused */

typedef struct SockCtrlHello {
     char ipAddr[SOCK_MAX_IP_LEN];
     Port udpCtrlPort;
     Port tcpCtrlPort;
} SockCtrlHello;

typedef struct SockCtrlFarewell {
     char ipAddr[SOCK_MAX_IP_LEN];
} SockCtrlFarewell;

typedef struct SockCtrlInitHandoff {
     char ipAddr[SOCK_MAX_IP_LEN];
} SockCtrlInitHandoff;

typedef struct SockCtrlDoHandoff {
     char ipAddr[SOCK_MAX_IP_LEN];
} SockCtrlDoHandoff;

typedef struct SockCtrlSetRate {
     int rate;
} SockCtrlSetRate;

typedef struct SockCtrlSetRemote {
     Port port;
} SockCtrlSetRemote;

typedef struct SockCtrlAck {
	int numBytes;
} SockCtrlAck;

typedef struct SockCtrlNewCtrlChannel {
     int protocol;
     char ipAddr[SOCK_MAX_IP_LEN];
     int port;
} SockCtrlNewCtrlChannel;

typedef struct SockCtrlPkt {
     int reqType;
     struct sockaddr_in srcAddr;
     union {
	  SockCtrlInitHandoff         	initHandoff;
	  SockCtrlDoHandoff           	doHandoff;
	  SockCtrlSetRate             	setRate;
	  SockCtrlSetRemote           	setRemote;
	  SockCtrlNewCtrlChannel  	newCtrlChannel;
	  SockCtrlFarewell		farewell;
	  SockCtrlHello			hello;
	  SockCtrlAck			ack;
     } data;
} SockCtrlPkt;


extern Sock_Status      Sock_RecvCtrl(Socket *ctrlSdPtr, Socket *sdPtr);
extern Sock_Status 	Sock_SendCtrl(Socket *sdPtr, RemoteHost *rhost, 
				      SockCtrlPkt *ctrlPktPtr);
extern Sock_Status      Sock_CtrlSetAddr(Socket *ctrlSdPtr, Port port);
extern Sock_Status      Sock_CtrlSetCtrlAddr(Socket *ctrlSdPtr, Port port);
extern Sock_Status      Sock_CtrlSetCtrlPort(Socket *ctrlSdPtr, Port port);
extern char             *BroadcastAddr(char *normAddr);
extern int              IsIPAddr(char *id, int length);

