#ifndef VROUTEH
#define VROUTEH
#include <kvm.h>
#include <netinet/in.h>
#include "easysocklib.h"
#include "decap.h"
#include "sample.h"
#include "beacon.h"

#define TRUE            1
#define FALSE           0
#define MAX_BS 		8	/* per interface */
#define MAX_SAMPLES 	4	/* number of beacon samples for decision */
#define TIMER_RATE	5
#define BEACON_TIMEOUT  100000
#define MAX_INTERFACES  8	/* should be enough */
#define MAX_IFNAME      20	/* interface string, e.g, wlp0 */
#define MAX_HOST_LEN    256

/* Base station states -- at most one can be forwarding per interface */
typedef enum {BS_INACTIVE=0, BS_BUFFERING, BS_FORWARDING, BS_DEAD} BS_States;

/* Different wireless interface types */
typedef enum {ETHERNET_TYPE, INFRARED_TYPE, WAVELAN_TYPE, METRICOM_TYPE, OTHER_TYPE} IfaceTypes;

/* Different reasons we could hand off */
typedef enum {DISCONNECTED=0, SIGNAL_STUFF, ADVICE} HandoffReason;

/* The state that vrouted is in with respect to choosing the right BS */
typedef enum {INACTIVE=0, NORMAL, WAITING} HandoffState;

/* Which type of data to forward--packet header or full packet */
typedef enum {HEADER, PACKET} PacketType;

typedef struct BSState {
	struct timeval lastBeaconTime;
	struct timeval prevLastBeaconTime;
	char wiName[20];		/* wired interface name for BS */
	char wlName[20];		/* wireless interface name for BS */
	BS_States state;		/* current state w.r.t mobile */
	PacketType hdrOnly;    /* Whether this bs is sending full packets or headers */
	Socket *sdPtr;
	struct in_addr wlAddr;	/* IP address of wireless interface */
	struct in_addr wiAddr;	/* IP address of wired interface */
	struct in_addr bAddr;   /* Beacon address for this interface */
	int lastSamp;		/* last sample in circular buffer of samples */
	WLSamp samples[MAX_SAMPLES]; /* circular buffer of samples */
	int numBest;		/* Number of times in a row that this BS has been the best */
	int foreignBS; 		/* If 1, this base station is advertising from afar for a local wireless net */
} BSState;

typedef struct IFace {		/* interface-specific data structure */
	struct timeval lastBeaconTime;
	struct timeval prevLastBeaconTime;
	BSState *oldBestBS;
	char ifname[MAX_IFNAME];	/* interface name, e.g wlp0, efp0, etc. */
	IfaceTypes iftype;		/* interface type */
	int  ifNum;		/* e.g, wlp0 has number 0 */
	int  active;		/* whether interface is active or not */
	char locName[MAX_HOST_LEN];/* hostname on wireless interface */
	struct in_addr locAddr;	/* local address of interface */
	struct in_addr netAddr;    /* network address */
	struct in_addr netMask;    /* subnet mask for the network */
	Socket *sdPtr;		/* socket descriptor for beacons */
	BSState *curBSPtr;		/* current BS for interface */
	BSState *BSPtr[MAX_BS];	/* array of different base stations */
	int beaconPort;		/* port to listen for beacons */
	int ifs;			/* interface socket */
	SampDesc *sampdPtr;	/* power sample (currently Wavelan only) */
	int numBufing;		/* number of buffering BSs for interface */
	int desBufing;		/* number of desired buffering base stations */
	int numBS;			/* number of BSs for interface */
	int powThresh;		/* power threshold for switching BSs */
	int dontUse;               /* 0 => can use, 1 => don't use */
} IFace;

typedef struct RouteState {
	int routes;                /* route socket */
	int    numIfaces;		/* total number of initialized interfaces */
	int    numActiveIfaces;	/* number of interfaces currently active */
	IFace  *iface[MAX_INTERFACES]; /* array of interface-specific pointers */
	IFace  *curiPtr;      /* A Pointer to the current interface */
	/* Note that this means that only 1 interface is actively being used */
	/* at a time. */
	struct in_addr realAddr;	/* real address of MH (invariant) */
	struct in_addr multiAddr;	/* multicast addr used by home agent for MH */
	int    state;
} RouteState;

extern void ClearOld(RouteState *sysStatePtr, IFace *iPtr, 
		     struct timeval *curTimePtr);
extern void AnalyzeState(RouteState *sysStatePtr);
extern int esSendUnencapReq(BSRTypes type, PacketType pkttype, BSState * bsPtr, struct in_addr locAddr,
			    struct in_addr realAddr, struct in_addr multiAddr,
			    struct mobileip_ids * idsPtr);
extern int GetInterface(IFace *iPtr);
extern IFace *GetBestIface(RouteState *sysStatePtr, BeaconMsg *beacons, 
			   BeaconMsg *bmsg);
extern SampDesc *InitSample(IFace *iPtr);
extern void ProcessUserRequest(char *usermsg, RouteState *sysStatePtr, IFace *curiPtr);
extern void DoHandoff(RouteState *sysStatePtr, IFace *curiPtr, IFace *newiPtr, BSState *oldBSPtr, BSState *newBSPtr);
extern void CancelUserCmds(RouteState *sysStatePtr);
extern void ForcedHandoff(int type, RouteState *sysStatePtr, IFace *curiPtr, char *newIfacename, struct in_addr *newBSaddr);

#endif


