#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <net/route.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define	NSIP
#include <netns/ns.h>
#include <netns/ns_if.h>
#include <netdb.h>

#define EON
#include <netiso/iso.h>
#include <netiso/iso_var.h>
#include <sys/protosw.h>

#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <dbgprintlib.h>
#include "easysocklib.h"
#include "wavelan.h"
#include "beacon.h"
extern int beaconPort;

#define SIOCSIFBEACON    _IOW('i',101, struct ifbeacon) /* register basestn */
#define SIOCGIFBEACON   _IOWR('i',102, struct ifbeacon) /* get snr measure */

void getetheraddr(addr, sdl)
	u_long addr;
	struct sockaddr_dl *sdl;
{
	int mib[6];
	size_t needed;
	char *lim, *buf, *next;
	struct rt_msghdr *rtm;
	struct sockaddr_inarp *sin;
	struct sockaddr_dl *sdl_t;

	mib[0] = CTL_NET;
	mib[1] = PF_ROUTE;
	mib[2] = 0;
	mib[3] = AF_INET;
	mib[4] = NET_RT_FLAGS;
	mib[5] = RTF_LLINFO;
	if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0)
		return;
	if ((buf = malloc(needed)) == NULL)
		return;
	if (sysctl(mib, 6, buf, &needed, NULL, 0) < 0)
		return;
	lim = buf + needed;
	for (next = buf; next < lim; next += rtm->rtm_msglen) {
		rtm = (struct rt_msghdr *)next;
		sin = (struct sockaddr_inarp *)(rtm + 1);
		sdl_t = (struct sockaddr_dl *)(sin + 1);
		if (addr == sin->sin_addr.s_addr) {
			bcopy(sdl_t, sdl, sizeof(struct sockaddr_dl));
			break;
		}
	}
	
}

void setifbeacon(int ifs, char *addr, char *iface, char cmd)
{
	struct ifbeacon ifb;
	u_long val;
	struct sockaddr_dl sdl;
	struct hostent *hp;
	struct sockaddr_inarp sin;
	int count;
	u_long cmd2;
	
	if ((val = inet_addr(addr)) != -1)
		ifb.beacon.ip_addr.s_addr = val;
	else if ((hp = gethostbyname(addr))) {
		bcopy((char *)hp->h_addr, (char *)&(sin.sin_addr),
		      sizeof(sin.sin_addr));
		ifb.beacon.ip_addr.s_addr = sin.sin_addr.s_addr;
	}
	getetheraddr(ifb.beacon.ip_addr, &sdl);
	/* Try 3 times to get an arp entry */
	count=0;
	while (!sdl.sdl_alen) {
		count++;
		printf("ifconfig: no arp entry for %s\n",addr);
		SendBeaconReq(addr, beaconPort);
		getetheraddr(ifb.beacon.ip_addr, &sdl);
		if (count==3){
			/* Give up */
			return;
		}
	}
	
	bcopy(LLADDR(&sdl), ifb.beacon.eth_addr, 
	      sizeof(ifb.beacon.eth_addr));
	ifb.beacon.siglvl = 0;
	ifb.beacon.sillvl = 0;
	ifb.beacon.sigqual = 0;
	ifb.cmd = cmd;
	dprintf(10,"setifbeacon: %sing %s %x as %x:%x:%x:%x:%x:%x\n",
	       cmd == ADD_BEACON ? "add" : "del",
	       addr, ifb.beacon.ip_addr.s_addr,
	       ifb.beacon.eth_addr[0], ifb.beacon.eth_addr[1],
	       ifb.beacon.eth_addr[2], ifb.beacon.eth_addr[3],
	       ifb.beacon.eth_addr[4], ifb.beacon.eth_addr[5]);
	strncpy(ifb.ifb_name, iface, sizeof (ifb.ifb_name));
	cmd2=SIOCSIFBEACON;
	if (ioctl(ifs, cmd2, (caddr_t)&ifb) < 0)
		perror("ioctl (set beacon)");

	return;
}

void DelBeacon(int ifs, char *addr, char *iface)
{
	setifbeacon(ifs, addr, iface, DEL_BEACON);
}
	
void AddBeacon(int ifs, char *addr, char *iface)
{
	setifbeacon(ifs, addr, iface, ADD_BEACON);
}
	
int
SendBeaconReq(char *addr, int port)
{
	Socket *sdPtr;
	BeaconMsg bmsg;
	int temp;
	struct sockaddr_in locAddr;
	struct hostent *hp;

	sdPtr = Sock_Socket(SOCK_UDP);
	Sock_SetCompatible(sdPtr, 1);
	if (Sock_Connect(sdPtr, addr, port) == SOCK_FAILURE)
		return(-1);
	bmsg.type = BEACON_REQ;
	if ((locAddr.sin_addr.s_addr = inet_addr(addr)) == -1)
		if ((hp = gethostbyname(addr))) {
			bcopy((char *)hp->h_addr, (char *)&(locAddr.sin_addr),
			      sizeof(locAddr.sin_addr));
		}
	bmsg.data.req.addr.s_addr = locAddr.sin_addr.s_addr;
	gettimeofday(&bmsg.curTime, NULL);
	temp = Sock_Send(sdPtr, (char *)&bmsg, sizeof(BeaconMsg));
	Sock_Close(sdPtr);
	Sock_Free(sdPtr);

	return(0);
}

int InitBeacon(char *iface)
{
     int ifs;
     int metric;
     struct ifreq ifr;
     struct ifbeacon ifb;
	
     strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
     ifs = socket(AF_INET, SOCK_DGRAM, 0);
     if (ifs < 0) {
	  perror("InitBeacon: socket failed");
	  return(-1);
     }
     if (ioctl(ifs, SIOCGIFFLAGS, (caddr_t)&ifr) < 0) {
	  perror("InitBeacon: ioctl (SIOCGIFFLAGS)");
	  return(-1);
     }
     ifb.beacon.siglvl = 0;
     ifb.beacon.sillvl = 0;
     ifb.beacon.sigqual = 0;
     ifb.cmd = DEL_ALL_BEACONS;
     strncpy(ifb.ifb_name, iface, sizeof (ifb.ifb_name));
     if (ioctl(ifs, SIOCSIFBEACON, (caddr_t)&ifb) < 0)
	     perror("ioctl (del all beacons)");
     strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
     if (ioctl(ifs, SIOCGIFMETRIC, (caddr_t)&ifr) < 0)
	  perror("ioctl (SIOCGIFMETRIC)");
     else
	  metric = ifr.ifr_metric;
     return(ifs);
}
