#include <sys/types.h>
#include <sys/socket.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "easysocklib.h"
#include "option.h"
#include "dbgprintlib.h"

#include "wavelan.h"
#include "beacon.h"
#include "conf.h"

int debug = 0;
char *confFileName = "/etc/daedalus.conf";
Port beaconPort = BEACON_PORT;
int beaconDelay = 1;
char *beaconAddrs = "204.161.78.159";
char *wiName = "ajanta";
char *wlNames = "ajanta-wl";

Option optionArray[] = {
  {OPT_INT, "debug", (char*)&debug,
     "Turn on debugging (higher value -> more debugging)."},
  {OPT_STRING, "confFileName", (char *)&confFileName,
   "Configuration file for paramaters (default /etc/daedalus.conf)."},
  {OPT_INT, "beaconPort", (char *)&beaconPort,
     "port of beacon packets"},
  {OPT_INT, "beaconDelay", (char *)&beaconDelay,
     "time in sec between beacons"},
  {OPT_STRING, "beaconAddrs", (char *)&beaconAddrs,
     "broadcast addrs on beaconed net"},
  {OPT_STRING, "wiName", (char *)&wiName,
     "wired name for bs"},
  {OPT_STRING, "wlNames", (char *)&wlNames,
     "wireless names for bs"}
};

int main(int argc, char *argv[])
{
	Socket *beaconSocks[MAX_WIRELESS_IFS];
	BeaconMsg bmsgs[MAX_WIRELESS_IFS];
	struct hostent *hp;
	int i=0, temp, numWlNames, numBeaconAddrs, type, typeSize;
	struct in_addr wiAddr, wlAddrs[MAX_WIRELESS_IFS];
	char *bAddrs[MAX_WIRELESS_IFS];	
	char *tmpPtr;
	char *tmpBeaconAddr;
	char *tmpWlName;
	char errStr[1000];
	
	/* The argument ParseConfFile expects that any string pointers
	   passed as arguments have room for a strcpy() of a new value.
	   So we malloc() a new string and copy the old value to this
	   string. */
	tmpPtr=beaconAddrs;
	beaconAddrs=malloc(100*sizeof(char));
	strcpy(beaconAddrs, tmpPtr);
	
	tmpPtr=wiName;
	wiName=malloc(100*sizeof(char));
	strcpy(wiName, tmpPtr);

	tmpPtr=wlNames;
	wlNames=malloc(100*sizeof(char));
	strcpy(wlNames, tmpPtr);

	/* Use the values in the configuration file for the beacon
           port, delay between beacon packets, address for beacon
           packets, wired name, and wireless name. */
	ParseConfFile(confFileName, 5,
		      "beaconPort", "%d", &beaconPort,
		      "beaconDelay", "%d", &beaconDelay,
		      "beaconAddrs", "%[^\n]", beaconAddrs,
		      "wiName", "%s", wiName,
		      "wlNames", "%[^\n]", wlNames);

	/* Command-line options override those in the configutration file */
	Opt_Parse (argc, argv, optionArray, Opt_Number(optionArray), 0);
	SetDebugLevel(debug);

	/* The addresses given may be names or addresses. If they are names, convert them to addresses. */
	if ((wiAddr.s_addr = inet_addr(wiName)) == -1) {
		if ((hp = gethostbyname(wiName))) {
			memcpy((char *)&wiAddr, (char *)hp->h_addr,
			       sizeof(wiAddr));
		} else {
			perror("wiAddr:");
			exit(-1);

		}
	}

	/* Split up the list of beacon addresses and wireless names */
	/* The addresses given may be names or addresses. If they are names, convert them to addresses. */
	numWlNames=0;
	while ((tmpWlName = strsep(&wlNames, " ")) != NULL){
		if ((wlAddrs[numWlNames].s_addr = inet_addr(tmpWlName)) == -1) {
			if ((hp = gethostbyname(tmpWlName))) {
				memcpy((char *)&wlAddrs[numWlNames], (char *)hp->h_addr,
				       sizeof(wlAddrs[numWlNames]));
			} else {
				sprintf(errStr, "While converting wlAddr %s", tmpWlName);
				perror(errStr);
				exit(-1);
			}
		}
		numWlNames++;
	}
	
	numBeaconAddrs=0;
	while ((tmpBeaconAddr = strsep(&beaconAddrs, " ")) != NULL){
		bAddrs[numBeaconAddrs]=malloc((strlen(tmpBeaconAddr)+1)*sizeof(char));
		strcpy(bAddrs[numBeaconAddrs], tmpBeaconAddr);
		numBeaconAddrs++;
	}
	
	/* There should be one wireless name for each beacon address */
	if (numWlNames!=numBeaconAddrs){
		fprintf(stderr, "Number of beacon address != num wireless addresses, exiting.\n");
		exit(-1);
	}
	
	/* For each beacon address, open a socket to the beacon address */
	for(i=0; i<numBeaconAddrs; i++){
		beaconSocks[i] = Sock_Socket(SOCK_UDP);
		Sock_SetCompatible(beaconSocks[i], 1);
		Sock_SetBufferSize(beaconSocks[i], 16384);
		if ((temp = Sock_Connect(beaconSocks[i], bAddrs[i], beaconPort)) == SOCK_FAILURE)
			exit(-1);

		type = IPTOS_LOWDELAY;
		typeSize = sizeof(type);
		if (setsockopt(Sock_GetFD(beaconSocks[i]), IPPROTO_IP, IP_TOS, 
			       (char *)&type,typeSize) < 0)
			perror("Sock_Socket: ip tos");

		bmsgs[i].type = BEACON;
		bmsgs[i].data.beacon.wlAddr.s_addr = wlAddrs[i].s_addr;
		bmsgs[i].data.beacon.wiAddr.s_addr = wiAddr.s_addr;
		bmsgs[i].data.beacon.bAddr.s_addr = inet_addr(bAddrs[i]);
	}
	
	/* main loop:
	   send a beacon packet,
	   sleep
	*/
	while (1) {
		for(i=0; i<numBeaconAddrs; i++){
			gettimeofday(&bmsgs[i].curTime, NULL);
			temp = Sock_Send(beaconSocks[i], (char *)&bmsgs[i], sizeof(BeaconMsg));
			if (temp == SOCK_FAILURE) {
				exit(-1);
			}
			if (temp == SOCK_FD_CLOSED) {
				printf("remote closed\n");
				exit(-1);
			}
		}
		sleep(beaconDelay);
	}

	/* Never Get Here */
}
