#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>
#include <kvm.h>
#include <sys/file.h>
#include <nlist.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include "in.h"
#include <netinet/if_ether.h>
#include <net/route.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_var.h>

#include "ip_mobile.h"
#include "option.h"

int sm = 0;
int cm = 0;
int sh = 0;
int st = 0;
int ad = 0;
int cd = 0;
int ch = 0;
int ia = 0;
int r  = 0;
int l  = 0;
int ifon = 0;
int ifoff = 0;
int bthresh = 0;
int bpid = 0;
char *saddr = "204.161.78.34";
char *ifaddr = "204.161.78.36";

Option optionArray[] = {
{OPT_TRUE, "sm", (char *)&sm,
         "set mh address"},
{OPT_TRUE, "cm", (char *)&cm,
         "clear mh address"},
{OPT_TRUE, "st", (char *)&st,
         "set tunnel address"},
{OPT_TRUE, "ad", (char *)&ad,
         "add don't touch address"},
{OPT_TRUE, "cd", (char *)&cd,
         "clear don't touch address"},
{OPT_TRUE, "ifon", (char *)&ifon,
	 "Allow packets through if"},
{OPT_TRUE, "ifoff", (char *)&ifoff,
	 "Disallow packets through if"},
{OPT_INT, "bthresh", (char *)&bthresh,
	 "Number of packets via 1 interface before HC is notified"},
{OPT_INT, "bpid", (char *)&bpid,
	 "Process ID of process to be notified when burst happens"},
{OPT_TRUE, "r", (char *)&r,
	 "reset"},
{OPT_TRUE, "l", (char *)&l,
	 "list unencap state in kernel"},
{OPT_STRING, "s", (char *)&saddr,
	 "unicast addr"},
{OPT_STRING, "i", (char *)&ifaddr,
	 "output interface addr (string when used w/ ifon, ifoff)"},
};

struct nlist nl[] = {
#define	N_MIPSTATE	0
	{ "_mip_state" },
	{""},
};

char *memf = NULL;
char *nlistf = NULL;
kvm_t *kvmd;

char *InetNtoa(struct in_addr addr)
{
        char *s = inet_ntoa(addr);
        char *p = (char *)malloc(1000);
        strcpy(p, s);
		p[strlen(s)+1]='\0';
        return p;
}

/*
 * Read kernel memory, return 0 on success.
 */
int
kread(addr, buf, size)
	u_long addr;
	char *buf;
	int size;
{

	if (kvm_read(kvmd, addr, buf, size) != size) {
		/* XXX this duplicates kvm_read's error printout */
		(void)fprintf(stderr, "kvm_read %s\n", 
		    kvm_geterr(kvmd));
		return (-1);
	}
	return (0);
}

void GetState(int state, char *string)
{
	 switch (state) {
	 case 0:
		  strcpy(string, "MIP_UNUSED");
		  break;
	 case 1:
		  strcpy(string, "MIP_UNENCAP_FWD");
		  break;
	 case 2:
		  strcpy(string, "MIP_UNENCAP_BUF");
		  break;
	 case 3:
		  strcpy(string, "MIP_ENCAP");
		  break;
	 }
}

#define Dt_State(x) ((x==0)? "MIP_DT_USED": "MIP_DT_UNUSED")

void PrintMHState(u_long stateaddr)
{
	struct mobilemh_state mst;
	int i;
	
	kread(stateaddr, (char *)&mst, sizeof(struct mobilemh_state));

	printf("Blocked Interface=%s\n", (char *) mst.mipBlockedIfs);
	printf("Handoff Controller PID=%d\n", mst.mipHandoffCtlrPid);
	printf("Burst Threshold=%d\n", mst.mipBurstThreshold);
	printf("Num Consec Packets=%d\n", mst.mipBurstCount);
	printf("Burst is from %s\n", mst.mipBurstIf);
	printf("Inserted address is %s\n", inet_ntoa(mst.mip_mh_addr));
	printf("Don't use addrs:\n");
	for(i=0; i<mst.num_dt; i++){
		printf("%d: %s %s\n", i, Dt_State(mst.dt[i].state), inet_ntoa(mst.dt[i].addr));
	}
}

void main(int argc, char *argv[])
{
     int fd, j; 
     struct ip_mobreq req;
     char buf[255];
     int option=0;

     Opt_Parse (argc, argv, optionArray, Opt_Number(optionArray), 0);

     /* setup basestation stats retrieval */
     if ((kvmd = kvm_openfiles(nlistf, memf, NULL, O_RDONLY, buf)) == NULL) {
	     fprintf(stderr, "kvm_open: %s\n", buf);
	     exit(1);
     }
     if (kvm_nlist(kvmd, nl) < 0 || nl[0].n_type == 0) {
	     if (nlistf)
		     fprintf(stderr, "%s: no namelist\n", nlistf);
	     else
		     fprintf(stderr, "no namelist\n");
	     exit(1);
     }

     if (l) {
	     PrintMHState(nl[N_MIPSTATE].n_value);
     }

     req.imr_ifaddr.s_addr = inet_addr(ifaddr);
     req.imr_ins_addr.s_addr = inet_addr(saddr);

     fd = socket(AF_INET, SOCK_STREAM, 0);
     
     j = sizeof(req);
     if (sm) 
	     option = IP_SET_MH_ADDR;
     if (cm) 
	     option = IP_CLR_MH_ADDR;
     if (ad) 
	     option = IP_ADD_DT_ADDR;
     if (cd) 
	     option = IP_CLR_DT_ADDR;
     if (sh) 
	     option = IP_TUNNEL_ADDR;
     if (ifon){
	     option = IP_IF_ON;
	     strcpy(req.blockedIf, ifaddr);
     }
     if (ifoff){
	     option = IP_IF_OFF;
	     strcpy(req.blockedIf, ifaddr);
     }
     if (bthresh) {
	     option = IP_NOTIFY_THRESHOLD;
	     req.burstThreshold=bthresh;
     }
     if (bpid) {
	     option = IP_NOTIFY_PID;
	     req.handoffCtlrPid=bpid;
     }
     if (r) 
	     option = IP_CLR_ALL;
     
     if (option)
	     if (setsockopt(fd,IPPROTO_IP, option,(char *)&req,j) < 0){
		     fprintf(stderr, "While trying option %d...", option);
		     perror("setsockopt");
	     }
}





