#include <sys/param.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/mbuf.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/route.h>
#include <net/if_dl.h>
#include <netinet/in.h>
#include <netns/ns.h>
#include <netiso/iso.h>
#include <netccitt/x25.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <paths.h>

extern int debug;
#define ROUNDUP(a) \
	((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
#define NEXTADDR(u) \
	    l = ROUNDUP(u.sa.sa_len); bcopy((char *)&(u), cp, l); cp += l;
#define rtm m_rtmsg.m_rtm

void DelDefaultRoute(int routes)
{
	int rlen;
	char *err;
	union	sockunion {
		struct	sockaddr sa;
		struct	sockaddr_in sin;
		struct	sockaddr_ns sns;
		struct	sockaddr_iso siso;
		struct	sockaddr_dl sdl;
		struct	sockaddr_x25 sx25;
	} so_dst, so_mask;
	struct {
		struct	rt_msghdr m_rtm;
		char	m_space[512];
	} m_rtmsg;
	static int seq, l;
	register char *cp = m_rtmsg.m_space;

	memset(&m_rtmsg, 0, sizeof(m_rtmsg));
	memset(&so_dst, 0, sizeof(so_dst));
	memset(&so_mask, 0, sizeof(so_mask));
	so_dst.sa.sa_family = AF_INET;
	so_mask.sa.sa_family = 0;
	so_dst.sa.sa_len = sizeof(struct sockaddr_in);
	so_mask.sa.sa_len = 0;

	rtm.rtm_type = RTM_DELETE;
	rtm.rtm_flags = RTF_STATIC | RTF_UP | RTF_GATEWAY;
	rtm.rtm_version = RTM_VERSION;
	rtm.rtm_seq = ++seq;
	rtm.rtm_addrs = RTA_DST | RTA_NETMASK;

	NEXTADDR(so_dst);
	NEXTADDR(so_mask);
	rtm.rtm_msglen = l = cp - (char *)&m_rtmsg;
	if ((rlen = write(routes, (char *)&m_rtmsg, l)) < 0) {
		switch (errno) {
		case ESRCH:
			err = "not in table";
			break;
		case EBUSY:
			err = "entry in use";
			break;
		case ENOBUFS:
			err = "routing table overflow";
			break;
		default:
			err = strerror(errno);
			break;
		}
		printf("defroute: %s\n", err);
	}		
}


void AddDefaultRoute(char *addr, int routes)
{
	int val, rlen;
	char *err;
	struct hostent *hp = 0;
	struct sockaddr_in sin;
	union	sockunion {
		struct	sockaddr sa;
		struct	sockaddr_in sin;
		struct	sockaddr_ns sns;
		struct	sockaddr_iso siso;
		struct	sockaddr_dl sdl;
		struct	sockaddr_x25 sx25;
	} so_dst, so_gate, so_mask;
	struct {
		struct	rt_msghdr m_rtm;
		char	m_space[512];
	} m_rtmsg;
	static int seq,l;
	register char *cp = m_rtmsg.m_space;

	memset(&m_rtmsg, 0, sizeof(m_rtmsg));
	memset(&so_dst, 0, sizeof(so_dst));
	memset(&so_mask, 0, sizeof(so_mask));
	so_dst.sa.sa_family = so_gate.sa.sa_family = AF_INET;
	so_mask.sa.sa_family = 0;
	so_dst.sa.sa_len = so_gate.sa.sa_len = sizeof(struct sockaddr_in);
	so_mask.sa.sa_len = 0;
	if ((val = inet_addr(addr)) != -1)
		so_gate.sin.sin_addr.s_addr = val;
	else if ((hp = gethostbyname(addr))) {
		bcopy((char *)hp->h_addr, (char *)&(sin.sin_addr),
		      sizeof(sin.sin_addr));
		so_gate.sin.sin_addr.s_addr = sin.sin_addr.s_addr;
	}

	rtm.rtm_type = RTM_ADD;
	rtm.rtm_flags = RTF_STATIC | RTF_UP | RTF_GATEWAY;
	rtm.rtm_version = RTM_VERSION;
	rtm.rtm_seq = ++seq;
	rtm.rtm_addrs = RTA_DST | RTA_GATEWAY | RTA_NETMASK;

	NEXTADDR(so_dst);
	NEXTADDR(so_gate);
	NEXTADDR(so_mask);
	rtm.rtm_msglen = l = cp - (char *)&m_rtmsg;
	if ((rlen = write(routes, (char *)&m_rtmsg, l)) < 0) {
		switch (errno) {
		case ESRCH:
			err = "not in table";
			break;
		case EBUSY:
			err = "entry in use";
			break;
		case ENOBUFS:
			err = "routing table overflow";
			break;
		default:
			err = strerror(errno);
			break;
		}
		printf("defroute: %s\n", err);
	}		
}

void SetDefaultRoute(char *addr, int routes)
{
	DelDefaultRoute(routes);
	AddDefaultRoute(addr, routes);
}


void AddHostRoute(char *dst, char *gw, int routes)
{
	int val, rlen;
	char *err;
	struct hostent *hp = 0;
	struct sockaddr_in sin;
	union	sockunion {
		struct	sockaddr sa;
		struct	sockaddr_in sin;
		struct	sockaddr_ns sns;
		struct	sockaddr_iso siso;
		struct	sockaddr_dl sdl;
		struct	sockaddr_x25 sx25;
	} so_dst, so_gate;
	struct {
		struct	rt_msghdr m_rtm;
		char	m_space[512];
	} m_rtmsg;
	static int seq,l;
	register char *cp = m_rtmsg.m_space;

	memset(&m_rtmsg, 0, sizeof(m_rtmsg));
	memset(&so_dst, 0, sizeof(so_dst));
	memset(&so_gate, 0, sizeof(so_gate));
	so_dst.sa.sa_family = so_gate.sa.sa_family = AF_INET;
	so_dst.sa.sa_len = so_gate.sa.sa_len = sizeof(struct sockaddr_in);

	if ((val = inet_addr(gw)) != -1)
		so_gate.sin.sin_addr.s_addr = val;
	else if ((hp = gethostbyname(gw))) {
		bcopy((char *)hp->h_addr, (char *)&(sin.sin_addr),
		      sizeof(sin.sin_addr));
		so_gate.sin.sin_addr.s_addr = sin.sin_addr.s_addr;
	}

	if ((val = inet_addr(dst)) != -1)
		so_dst.sin.sin_addr.s_addr = val;
	else if ((hp = gethostbyname(dst))) {
		bcopy((char *)hp->h_addr, (char *)&(sin.sin_addr),
		      sizeof(sin.sin_addr));
		so_dst.sin.sin_addr.s_addr = sin.sin_addr.s_addr;
	}

	rtm.rtm_type = RTM_ADD;
	rtm.rtm_flags = RTF_STATIC | RTF_UP | RTF_GATEWAY | RTF_HOST;
	rtm.rtm_version = RTM_VERSION;
	rtm.rtm_seq = ++seq;
	rtm.rtm_addrs = RTA_DST | RTA_GATEWAY;

	NEXTADDR(so_dst);
	NEXTADDR(so_gate);
	rtm.rtm_msglen = l = cp - (char *)&m_rtmsg;
	if ((rlen = write(routes, (char *)&m_rtmsg, l)) < 0) {
		switch (errno) {
		case ESRCH:
			err = "not in table";
			break;
		case EBUSY:
			err = "entry in use";
			break;
		case ENOBUFS:
			err = "routing table overflow";
			break;
		default:
			err = strerror(errno);
			break;
		}
		printf("adding route: %s\n", err);
	}		
}

void DelHostRoute(char *dst, char *gw, int routes)
{
	int val, rlen;
	char *err;
	struct hostent *hp = 0;
	struct sockaddr_in sin;
	union	sockunion {
		struct	sockaddr sa;
		struct	sockaddr_in sin;
		struct	sockaddr_ns sns;
		struct	sockaddr_iso siso;
		struct	sockaddr_dl sdl;
		struct	sockaddr_x25 sx25;
	} so_dst, so_gate;
	struct {
		struct	rt_msghdr m_rtm;
		char	m_space[512];
	} m_rtmsg;
	static int seq,l;
	register char *cp = m_rtmsg.m_space;

	memset(&m_rtmsg, 0, sizeof(m_rtmsg));
	memset(&so_dst, 0, sizeof(so_dst));
	memset(&so_gate, 0, sizeof(so_gate));
	so_dst.sa.sa_family = so_gate.sa.sa_family = AF_INET;
	so_dst.sa.sa_len = so_gate.sa.sa_len = sizeof(struct sockaddr_in);

	if ((val = inet_addr(gw)) != -1)
		so_gate.sin.sin_addr.s_addr = val;
	else if ((hp = gethostbyname(gw))) {
		bcopy((char *)hp->h_addr, (char *)&(sin.sin_addr),
		      sizeof(sin.sin_addr));
		so_gate.sin.sin_addr.s_addr = sin.sin_addr.s_addr;
	}

	if ((val = inet_addr(dst)) != -1)
		so_dst.sin.sin_addr.s_addr = val;
	else if ((hp = gethostbyname(dst))) {
		bcopy((char *)hp->h_addr, (char *)&(sin.sin_addr),
		      sizeof(sin.sin_addr));
		so_dst.sin.sin_addr.s_addr = sin.sin_addr.s_addr;
	}

	rtm.rtm_type = RTM_DELETE;
	rtm.rtm_flags = RTF_STATIC | RTF_UP | RTF_GATEWAY | RTF_HOST;
	rtm.rtm_version = RTM_VERSION;
	rtm.rtm_seq = ++seq;
	rtm.rtm_addrs = RTA_DST | RTA_GATEWAY;

	NEXTADDR(so_dst);
	NEXTADDR(so_gate);
	rtm.rtm_msglen = l = cp - (char *)&m_rtmsg;
	if ((rlen = write(routes, (char *)&m_rtmsg, l)) < 0) {
		switch (errno) {
		case ESRCH:
			err = "not in table";
			break;
		case EBUSY:
			err = "entry in use";
			break;
		case ENOBUFS:
			err = "routing table overflow";
			break;
		default:
			err = strerror(errno);
			break;
		}
		printf("deleting route: %s\n", err);
	}		
}

int InitRoute()
{
     int routes;

     if ((routes = socket(PF_ROUTE, SOCK_RAW, 0)) < 0) {
	  perror("route socket");
	  return(-1);
     }
     shutdown(routes, 0); /* Don't want to read back our messages */
     return(routes);
}
			     
