
#include <sys/types.h>
#include <sys/protosw.h>
#include <sys/socket.h>

#include <net/if.h>
#include <net/if_types.h>
#include <net/if_dl.h>
#include <netinet/in.h>
#include <netinet/in_var.h>
#include <netinet/if_ether.h>
#include <netns/ns.h>
#include <netns/ns_if.h>
#include <netiso/iso.h>
#include <netiso/iso_var.h>
#include <arpa/inet.h>
#include <sys/file.h>

#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/device.h>
#include <kvm.h>
#include <nlist.h>
#include <limits.h>

#include "ip_mobile.h"
#include "wavelan.h"
#include "beacon.h"
#include "sample.h"
#include "vroute.h"

/* 
 * XXX This function is Wavelan-specific, especially its return value. It
 * is called even if the interface is not Wavelan. For now, it just returns 
 * NULL when this happens.
 */

SampDesc *InitSample(IFace *iPtr)
{
     struct nlist nl[] = {
#define N_WLCNTRS	0
     { "_wl_cntrs" },
	  "",
     };
     char *nlistf = NULL;
     char *memf = NULL;
     char buf[_POSIX2_LINE_MAX];
     SampDesc *newSDPtr;
     char *iface = iPtr->ifname;

     if (iPtr->iftype != WAVELAN_TYPE)
	  return 0;		/* not wavelan, return NULL for now */
     newSDPtr = (SampDesc *)calloc(sizeof(SampDesc), 1);
     if (iface[2] == 'p')
       nl[0].n_name = "_wlp_cntrs";
     else if ((iface[2] == 'g') && (iface[3] == 'p'))
       nl[0].n_name = "_wlgp_cntrs";

     if ((newSDPtr->kvmd = 
	  kvm_openfiles(nlistf, memf, NULL, O_RDONLY, buf)) == NULL) {
	  fprintf(stderr, "kvm_open: %s\n", buf);
	  return(0);
     }
     if (kvm_nlist(newSDPtr->kvmd, nl) < 0 || nl[0].n_type == 0) {
	  if (nlistf)
	       fprintf(stderr, "%s: no namelist\n", nlistf);
	  else
	       fprintf(stderr, "no namelist\n");
	  return(0);
     }
     newSDPtr->statsAddr = nl[N_WLCNTRS].n_value;
     return(newSDPtr);
}

/*
 * Print a description of the network interfaces.
 */
int
GetSample(SampDesc *sampdPtr, int ifNum, WLSamp *samplePtr)
{
	struct wl_cntrs wl_cntrs[NUM_IF];
	int i;

	kread(sampdPtr->statsAddr, (char *)&wl_cntrs, sizeof(wl_cntrs), 
	      sampdPtr->kvmd);

	for (i = 0; i < 8; i++) {
	     if (wl_cntrs[ifNum].beacons[i].ip_addr.s_addr == 
		 samplePtr->addr.s_addr){
		  dprintf(30, 
		      "%x:%x:%x:%x:%x:%x: SigLev= %d, SilLev=%d, SigQual=%d\n",
			  wl_cntrs[ifNum].beacons[i].eth_addr[0], 
			  wl_cntrs[ifNum].beacons[i].eth_addr[1],
			  wl_cntrs[ifNum].beacons[i].eth_addr[2],
			  wl_cntrs[ifNum].beacons[i].eth_addr[3],
			  wl_cntrs[ifNum].beacons[i].eth_addr[4],
			  wl_cntrs[ifNum].beacons[i].eth_addr[5],
			  wl_cntrs[ifNum].beacons[i].siglvl, 
			  wl_cntrs[ifNum].beacons[i].sillvl, 
			  wl_cntrs[ifNum].beacons[i].sigqual);
		  samplePtr->siglvl = wl_cntrs[ifNum].beacons[i].siglvl & 0x3f;
		  samplePtr->sillvl = wl_cntrs[ifNum].beacons[i].sillvl & 0x3f;
		  samplePtr->sigqual= wl_cntrs[ifNum].beacons[i].sigqual & 0xf;
		  break;
	     }   
        }
	if (wl_cntrs[ifNum].beacons[i].ip_addr.s_addr==samplePtr->addr.s_addr)
	     return (1);
	return(0);
}

