
/*
 * Copyright (c) 1995, 1996, 1997
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* 
 * Functions for advice handling and state reporting.
 * Original version written by Srinivasan Seshan (srini@watson.ibm.com)
 * modified heavily and released by Mark Stemm (stemm@cs.berkeley.edu)
 *
 * Daedalus group, Dept. of EECS, U.C. Berkeley.
 * Contact: stemm@cs.berkeley.edu, net@daedalus.cs.berkeley.edu
 */

#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <signal.h>
#include <unistd.h>

#include <net/if.h>

#include <netinet/in.h>
#include <netinet/in_var.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ip_mobile.h"
#include "easysocklib.h"
#include "option.h"
#include "dbgprintlib.h"

#include "wavelan.h"
#include "beacon.h"
#include "decap.h"
#include "sample.h"
#include "newroute.h"
#include "vroute.h"

extern Socket *state_sdPtr;

/* Possible requests are */
/* "ListActiveBS": List current BS */
   
/*		
 * Process a message from the user controller.
 *   advicemsg: the message received from the user controller
 *   sysSatePtr: pointer to the network state date structures
 *   curiPtr: pointer to the current interface
 */
void ProcessStateRequest(char *statemsg, RouteState * sysStatePtr, IFace * curiPtr)
{

	char replymsg[1000];
	
	if (strcmp(statemsg, "ListActiveBS")==0){
		sprintf(replymsg, "%s %s", inet_ntoa(sysStatePtr->realAddr), sysStatePtr->curiPtr->curBSPtr->wlName);
	} else {
		return;
	}
	
	if (Sock_Send(state_sdPtr, replymsg, strlen(replymsg))==SOCK_FAILURE){
		perror("Sock_Send()");
	}
}
